#include <math.h>
#include <stdint.h>
#include <string.h>
#include <zephyr.h>

#include "nrfx_i2s.h"

#define I2S_SCK_PIN 4
#define I2S_WS_PIN  5
#define I2S_SD_PIN  6

#define I2S_DATA_BLOCK_WORDS 512

static uint32_t m_buffer_rx[I2S_DATA_BLOCK_WORDS];
static nrfx_i2s_buffers_t initial_buffers;
static int counter = 0;

static void data_handler(nrfx_i2s_buffers_t const* p_released, uint32_t status) {
  counter++;
  if (NRFX_I2S_STATUS_NEXT_BUFFERS_NEEDED == status) {
    nrfx_err_t err = nrfx_i2s_next_buffers_set(&initial_buffers);
    if (err != 0) {  // TODO: Handle me
      k_panic();
    }
  }
  if (p_released) {
    if (p_released->p_rx_buffer != NULL) {
      // TODO: Handle buffers
    }
  }
}

static int mic_start_collecting_sound_level() {
  printk("Start collecting sound samples\n");
  memset(&m_buffer_rx, 0x00, sizeof(m_buffer_rx));
  initial_buffers.p_rx_buffer = m_buffer_rx;
  nrfx_i2s_config_t config = NRFX_I2S_DEFAULT_CONFIG(I2S_SCK_PIN, I2S_WS_PIN, NRFX_I2S_PIN_NOT_USED, NRFX_I2S_PIN_NOT_USED, I2S_SD_PIN);
  config.mode = NRF_I2S_MODE_MASTER;
  //config.mck_setup = NRF_I2S_MCK_DISABLED;
  config.mck_setup = NRF_I2S_MCK_32MDIV21;
  config.channels = NRF_I2S_CHANNELS_LEFT;
  config.sample_width = NRF_I2S_SWIDTH_24BIT;
  config.ratio = NRF_I2S_RATIO_96X;
  config.format = NRF_I2S_FORMAT_I2S;

  uint32_t err_code = nrfx_i2s_init(&config, data_handler);
  if (err_code != NRFX_SUCCESS) {
    k_panic();
  }
  // Crashes inside this
  nrfx_i2s_start(&initial_buffers, I2S_DATA_BLOCK_WORDS, 0);
  printk("Collection in progress\n");

  return 0;
}
void main() {
  printk("Main thread starting up\n");
  int err = mic_start_collecting_sound_level();
  if (err != 0) {
    printk("Fail: %d", err);
  }
  while (1) {
    printk("Counter: %d\n", counter);
    k_sleep(5000);
  }
}